use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007005
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( lib t xt ) );
__DATA__
BURNERSK
Champoux
Colored
Console
Dancer2
Etheridge
Fiegehenn
Guo
Julien
Karen
Logger
Steve
Yanick
burnersk
ether
lib
simbabque
steve
yanick
